//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//	ChatSelector.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "ChatSelector.h"
#include "packets.h"
#include "otherfunctions.h"
#include "Smtp.h"
#include "IP2Country.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif


CChatItem::CChatItem()
{
	m_pClient = NULL;
	m_pLog = NULL;
	m_strMessagePending.Empty();
	m_bNotify = false;
	m_iHistoryIndex = 0;
}

//	CChatSelector

IMPLEMENT_DYNAMIC(CChatSelector, CClosableTabCtrl)
CChatSelector::CChatSelector()
{
	lastemptyicon = false;
	blinkstate = false;
	m_Timer = 0;
}

CChatSelector::~CChatSelector()
{
}


BEGIN_MESSAGE_MAP(CChatSelector, CClosableTabCtrl)
	ON_WM_TIMER()
	ON_NOTIFY_REFLECT(TCN_SELCHANGE, OnTcnSelchangeChatsel)
	ON_WM_SIZE()
	ON_NOTIFY(EN_LINK, 124, OnEnLinkMessageBox)
	ON_WM_DESTROY()
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CChatSelector message handlers
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::Init()
{
	CRect		rcRect;

	GetClientRect(&rcRect);
	AdjustRect(false, rcRect);
	rcRect.DeflateRect(7, 7);
	ModifyStyle(0, WS_CLIPCHILDREN);
	chatout.Create(WS_VISIBLE | WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_NOHIDESEL, rcRect, this, 124);
	chatout.ModifyStyleEx(0, WS_EX_STATICEDGE, SWP_FRAMECHANGED);
	chatout.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	chatout.SetTitle(GetResString(IDS_CW_MESSAGES));
	chatout.m_dwFlags |= (HTC_ISLIMITED | HTC_ISWORDWRAP);
	chatout.AppendText(CLIENT_NAME, RGB(153,51,102), CLR_DEFAULT, HTC_LINK);
	chatout.AppendText(CString(" Version " CURRENT_VERSION_LONG " - ") + GetResString(IDS_CHAT_WELCOME) + _T("\n"), RGB(153,51,102));

	VERIFY((m_Timer = SetTimer(20, 1500, 0)));
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CChatItem* CChatSelector::StartSession(CUpDownClient* pClient, bool bForceFocus /* true */)
{
	if (pClient == NULL)
		return NULL;

	EMULE_TRY

	if (GetTabByClient(pClient) != (uint16)-1)
	{
		SetCurSel(GetTabByClient(pClient));
		if (bForceFocus)
			ShowChat();
		return NULL;
	}

	CChatItem		*pChatItem = new CChatItem();
	CRect			rcRect;

	pChatItem->m_pClient = pClient;
	pChatItem->m_pLog = new CHTRichEditCtrl;

	GetClientRect(&rcRect);
	AdjustRect(false, rcRect);
	rcRect.DeflateRect(7, 7);

	if (GetItemCount() == 0)
		rcRect.top += 19;

	pChatItem->m_pLog->Create(WS_CHILD | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_MULTILINE | ES_READONLY | ES_NOHIDESEL, rcRect, this, (UINT)-1);
	pChatItem->m_pLog->ModifyStyleEx(0, WS_EX_STATICEDGE, SWP_FRAMECHANGED);
	pChatItem->m_pLog->SendMessage(EM_SETMARGINS, EC_LEFTMARGIN | EC_RIGHTMARGIN, MAKELONG(3, 3));
	pChatItem->m_pLog->SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	pChatItem->m_pLog->m_dwFlags |= HTC_ISWORDWRAP;
	pChatItem->m_pLog->SetTargetDevice(NULL, 0);

	COleDateTime	timelog(COleDateTime::GetCurrentTime());
	CString			strName = pClient->GetUserName(), strCountry, strTemp;

	if (g_eMuleApp.m_pIP2Country->IsIP2Country())
		strCountry.Format(_T(" (%s)"), pClient->GetCountryName());
	strTemp.Format(_T("*** %s: %s (%s: %s:%u%s) - %s\n"), 
		GetResString(IDS_CHAT_START),
		strName,
		GetResString(IDS_IP), pClient->GetFullIP(), pClient->GetUserPort(), strCountry,
		timelog.Format());
	pChatItem->m_pLog->AppendText(strTemp, RGB(255, 0, 0));
	pClient->SetChatState(MS_CHATTING);
	if (pClient->IsFriend())
		pClient->m_pFriend->m_dwLastChatted = time(NULL);

	if (strName.GetLength() > 30)
	{
		strName.Truncate(30);
		strName += _T("...");
	}
	else if (strName.IsEmpty())
		strName.Format(_T("[%s]"), GetResString(IDS_UNKNOWN));

	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM | TCIF_TEXT | TCIF_IMAGE;
	tcitem.lParam = (LPARAM)pChatItem;
	tcitem.pszText = (TCHAR*)strName.GetString();
	tcitem.iImage = 0;
	
	int			iResult = InsertItem(GetItemCount(), &tcitem);

	g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlCloseButton.EnableWindow(true);
	g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlSendButton.EnableWindow(true);

	if (iResult != -1 && IsWindowVisible())
	{
		SetCurSel(iResult);
		pChatItem->m_pLog->SetTitle(pClient->GetUserName());
		if (bForceFocus)
			ShowChat();
	}

	return pChatItem;

	EMULE_CATCH

	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CChatSelector::GetTabByClient(CUpDownClient* client)
{
	TCITEM tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (GetItem(i, &tcitem) && ((CChatItem*)tcitem.lParam)->m_pClient == client)
			return i;
	}
	return (uint16)-1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CChatItem* CChatSelector::GetItemByClient(CUpDownClient* client)
{
	TCITEM	tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (GetItem(i, &tcitem) && ((CChatItem*)tcitem.lParam)->m_pClient == client)
			return(CChatItem*)tcitem.lParam;
	}
	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::ProcessMessage(CUpDownClient* sender, LPCTSTR message)
{
	CString		strResToken, strMessage = message;
	int			iCurPos = 0;

	strMessage.MakeLower();

	//filter me?
	if (g_eMuleApp.m_pGlobPrefs->IsCounterMeasures()
		&& (strMessage.Find(_T("emule sans upload @ http://www.chez.com/theworld/")) != -1
		|| strMessage.Find(_T("http://fullspeed.to/mison")) != -1
		|| strMessage.Find(_T("ketamine")) != -1))
	{
		AddDebugLogLine(RGB_LOG_DIMMED_TXT _T("Anti-leechermods: Client %s has been banned because of spamming"), sender->GetClientNameWithSoftware());
		sender->Ban(_T("AUTOMATED MESSAGE: Your client is being banned for spamming and leeching!"), GetResString(IDS_BAN_LEECHERSPAM));
		return;
	}

	CString		strFilter = g_eMuleApp.m_pGlobPrefs->GetMessageFilter();

	strFilter.MakeLower();
	for (;;)
	{
		strResToken = strFilter.Tokenize(_T("|"), iCurPos);
		if (strResToken.IsEmpty())
			break;
		if (strMessage.Find(strResToken) >= 0)
		{
			AddDebugLogLine(RGB_LOG_DIMMED_TXT _T("Filtered message '%s' from client %s"), message, sender->GetClientNameWithSoftware());
			return;
		}
	};

//	continue
	CChatItem		*pChatItem = GetItemByClient(sender);
	bool			bIsNewChatWindow = false;

	if (pChatItem == NULL)
	{
		if (GetItemCount() >= 50)
		{
			AddDebugLogLine(RGB_LOG_WARNING_TXT _T("Instant Messaging: Messages limit reached"));
			return;
		}

		if (g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 4) //no messages
			return;

		if ((g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 2) && !sender->IsFriend()) //only friends
			return;

		if ((g_eMuleApp.m_pGlobPrefs->GetAcceptMessagesFrom() == 3) && !sender->IsFriend()) //log non friends
		{
			AddLogLine(false, GetResString(IDS_IM_MSGFROMCHAT) + _T(" %s: %s"), sender->GetUserName(), message);
			sender->SetChatState(MS_NONE);
			return;
		}

		pChatItem = StartSession(sender);
		if (pChatItem == NULL)
			return;
		bIsNewChatWindow = true;
	}
	COleDateTime	timelog(COleDateTime::GetCurrentTime());

	strMessage.Format(_T("%s (%s): "), sender->GetUserName(), timelog.Format());
	pChatItem->m_pLog->AppendText(strMessage, RGB(50, 200, 250));
	pChatItem->m_pLog->AppendText(CString(message) + _T('\n'));

	if (g_eMuleApp.m_pGlobPrefs->GetAwayState())
	{
		if ((::GetTickCount() - pChatItem->m_pClient->GetAwayMessageResendCount()) > 3000)
		{ //send again only if 3 secs from last away message
			SendAwayMessage(pChatItem);
			pChatItem->m_pClient->SetAwayMessageResendCount(::GetTickCount());
		}
	}

	if ((iCurPos = GetTabByClient(sender)) != GetCurSel())
		SetItemState(iCurPos, TCIS_HIGHLIGHTED, TCIS_HIGHLIGHTED);
	if (!g_eMuleApp.m_pGlobPrefs->GetAwayState())
	{
		BOOL	bVisible = ::IsWindowVisible(::GetParent(m_hWnd));

	//	Show statusbar indicator if Messages window isn't active or our application isn't topmost one
		if (!bVisible || (g_eMuleApp.m_pdlgEmule->m_hWnd != ::GetForegroundWindow()))
		{
			pChatItem->m_bNotify = true;
		//	Send Notification is required
			if ( ( g_eMuleApp.m_pGlobPrefs->GetUseChatNotifier() &&
				(bIsNewChatWindow || g_eMuleApp.m_pGlobPrefs->GetNotifierPopsEveryChatMsg()) ) )
			{
				strMessage.Format(_T("%s %s:'%s'\n"), GetResString(IDS_TBN_NEWCHATMSG), sender->GetUserName(), message);
				g_eMuleApp.m_pSMTPConnection->SendMail(strMessage, true, g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled());
				g_eMuleApp.m_pdlgEmule->ShowNotifier(strMessage, TBN_CHAT, false, true);
			}
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CChatSelector::SendMessage(const CString& message)
{
	CChatItem		*pChatItem = GetCurrentChatItem();

	if (pChatItem == NULL || pChatItem->m_pClient == NULL)
		return false;

	if (pChatItem->m_strHistoryArray.GetCount() == g_eMuleApp.m_pGlobPrefs->GetMaxChatHistoryLines())
		pChatItem->m_strHistoryArray.RemoveAt(0);
	pChatItem->m_strHistoryArray.Add(CString(message));
	pChatItem->m_iHistoryIndex = pChatItem->m_strHistoryArray.GetCount();

	if (pChatItem->m_pClient->GetChatState() == MS_CONNECTING)
		return false;

#ifdef OLD_SOCKETS_ENABLED
	if (pChatItem->m_pClient->m_pRequestSocket && pChatItem->m_pClient->m_pRequestSocket->IsConnected())
	{
		SendMessagePacket(*pChatItem, message);

		COleDateTime	timelog(COleDateTime::GetCurrentTime());
		CString			strTmp;

		strTmp.Format(_T("%s (%s): "), g_eMuleApp.m_pGlobPrefs->GetUserNick(), timelog.Format());
		pChatItem->m_pLog->AppendText(strTmp, RGB(1, 180, 20));
		pChatItem->m_pLog->AppendText(message + _T('\n'));
	}
	else
	{
		pChatItem->m_pLog->AppendText(_T("*** ") + GetResString(IDS_CONNECTING), RGB(255, 0, 0));
		pChatItem->m_strMessagePending = message;
		pChatItem->m_pClient->SetChatState(MS_CONNECTING);
		pChatItem->m_pClient->TryToConnect();
	}
#endif //OLD_SOCKETS_ENABLED
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::ConnectingResult(CUpDownClient* sender, bool bSuccess)
{
	CChatItem		*pChatItem = GetItemByClient(sender);

	if (pChatItem == NULL || pChatItem->m_pClient == NULL)
		return;

	pChatItem->m_pClient->SetChatState(MS_CHATTING);

	if (!bSuccess)
	{
		if (!pChatItem->m_strMessagePending.IsEmpty())
		{
			pChatItem->m_pLog->AppendText(_T(' ') + GetResString(IDS_FAILED) + _T('\n'), RGB(255, 0, 0));
			pChatItem->m_strMessagePending.Empty();
		}
#ifdef _DEBUG
		else
		{
			pChatItem->m_pLog->AppendText(GetResString(IDS_CHATDISCONNECTED) + _T('\n'), RGB(255, 0, 0));
		}
#endif
	}
	else if (!pChatItem->m_strMessagePending.IsEmpty())
	{
		pChatItem->m_pLog->AppendText(_T(" ok\n"), RGB(255, 0, 0));
		SendMessagePacket(*pChatItem, pChatItem->m_strMessagePending);

		COleDateTime	timelog(COleDateTime::GetCurrentTime());
		CString			strTmp;

		strTmp.Format(_T("%s (%s): "), g_eMuleApp.m_pGlobPrefs->GetUserNick(), timelog.Format());
		pChatItem->m_pLog->AppendText(strTmp, RGB(1, 180, 20));
		pChatItem->m_pLog->AppendText(pChatItem->m_strMessagePending + _T('\n'));
		pChatItem->m_strMessagePending.Empty();
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::DeleteAllItems()
{
	TCITEM		tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (GetItem(i, &tcitem))
			delete (CChatItem*)tcitem.lParam;
	}
	chatout.ShowWindow(SW_SHOW);
	g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlCloseButton.EnableWindow(false);
	g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlSendButton.EnableWindow(false);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnTimer(UINT_PTR nIDEvent)
{
	blinkstate = !blinkstate;

	bool		globalnotify = false;
	TCITEM		tcitem;

	for (int i = 0; i < GetItemCount(); i++)
	{
		tcitem.mask = TCIF_PARAM;
		if (!GetItem(i, &tcitem))
			break;

		if (((CChatItem*)tcitem.lParam)->m_bNotify)
		{
			globalnotify = true;
			break;
		}
	}
	if (globalnotify)
	{
		g_eMuleApp.m_pdlgEmule->ShowMessageState(((blinkstate) ? 1 : 2));
		lastemptyicon = false;
	}
	else if (!lastemptyicon)
	{
		g_eMuleApp.m_pdlgEmule->ShowMessageState(0);
		lastemptyicon = true;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CChatItem* CChatSelector::GetCurrentChatItem()
{
	int			iCurSel = GetCurSel();

	if (iCurSel == -1)
		return NULL;

	TCITEM		tcitem;

	tcitem.mask = TCIF_PARAM;
	if (!GetItem(iCurSel, &tcitem))
		return NULL;
	return (CChatItem*)tcitem.lParam;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::ShowChat()
{
	int				iCurSel = GetCurSel();

	if (iCurSel == -1)
		return;

	bool			bWasChanged = (GetItemState(iCurSel, TCIS_HIGHLIGHTED) & TCIS_HIGHLIGHTED);

	SetItemState(iCurSel, TCIS_HIGHLIGHTED, NULL);

	CChatItem		*pChatItem = GetCurrentChatItem(), *pUpdateItem;

	if (pChatItem == NULL)
		return;

	pChatItem->m_pLog->SetRedraw(false);
	pChatItem->m_pLog->ShowWindow(SW_SHOW);
	if ((pChatItem->m_pLog->m_dwFlags & HTC_ISAUTOSCROLL) != 0 && bWasChanged)
		pChatItem->m_pLog->ScrollToLastLine();
	pChatItem->m_pLog->SetRedraw(true);

	TCITEM			tcitem;
	int				i = 0;

	tcitem.mask = TCIF_PARAM;
	while (GetItem(i++, &tcitem))
	{
		pUpdateItem = (CChatItem*)tcitem.lParam;
		if (pUpdateItem != pChatItem)
			pUpdateItem->m_pLog->ShowWindow(SW_HIDE);
	}

	pChatItem->m_pLog->SetTitle(pChatItem->m_pClient->GetUserName());
	pChatItem->m_bNotify = false;
	g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlInputText.SetFocus();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnTcnSelchangeChatsel(NMHDR *pNMHDR, LRESULT *pResult)
{
	ShowChat();
	*pResult = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
INT	CChatSelector::InsertItem(int nItem, TCITEM* pTabCtrlItem)
{
	if (GetItemCount() == 0)
		chatout.ShowWindow(SW_HIDE);

	int		iResult = CClosableTabCtrl::InsertItem(nItem, pTabCtrlItem);

	RedrawWindow();
	return iResult;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CChatSelector::DeleteItem(int nItem)
{
	CClosableTabCtrl::DeleteItem(nItem);
	if (GetItemCount() == 0)
	{
		chatout.ShowWindow(SW_SHOW);
		g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlCloseButton.EnableWindow(false);
		g_eMuleApp.m_pdlgEmule->m_wndChat.m_ctlSendButton.EnableWindow(false);
	}
	RedrawWindow();
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::EndSession(CUpDownClient* client, bool bForceFocus /* true */)
{
	int		iCurSel;

	if (client)
		iCurSel = GetTabByClient(client);
	else
		iCurSel = GetCurSel();

	if (iCurSel == -1)
		return;

	TCITEM		tcitem;
	
	tcitem.mask = TCIF_PARAM;

	if (!GetItem(iCurSel, &tcitem) || tcitem.lParam == 0)
		return;

	CChatItem		*pChatItem = (CChatItem*)tcitem.lParam;

	pChatItem->m_pClient->SetChatState(MS_NONE);

	DeleteItem(iCurSel);
	delete pChatItem;

	int			iTabItems = GetItemCount();

	if (iTabItems > 0)
	{
		// select next tab
		if (iCurSel == CB_ERR)
			iCurSel = 0;
		else if (iCurSel >= iTabItems)
			iCurSel = iTabItems - 1;
		(void)SetCurSel(iCurSel);				// returns CB_ERR if error or no prev. selection(!)
		iCurSel = GetCurSel();					// get the real current selection
		if (iCurSel == CB_ERR)					// if still error
			iCurSel = SetCurSel(0);
		if (bForceFocus)
			ShowChat();
	}

}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnSize(UINT nType, int cx, int cy)
{
	CClosableTabCtrl::OnSize(nType, cx, cy);

	CRect		rcRect;

	GetClientRect(&rcRect);
	AdjustRect(false, rcRect);
	rcRect.DeflateRect(7, 7);

	if (GetItemCount() > 0)
		rcRect.top -= 19;

	chatout.SetWindowPos(NULL, rcRect.left, rcRect.top, rcRect.Width(), rcRect.Height(), SWP_NOZORDER);

	TCITEM			tcitem;
	CChatItem		*pChatItem;
	int				i = 0;

	tcitem.mask = TCIF_PARAM;
	while (GetItem(i++, &tcitem))
	{
		pChatItem = (CChatItem*)tcitem.lParam;
		pChatItem->m_pLog->SetWindowPos(NULL, rcRect.left, rcRect.top + 19, rcRect.Width(), rcRect.Height() - 19, SWP_NOZORDER);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CChatSelector::PreTranslateMessage(MSG* pMsg)
{
	return CClosableTabCtrl::PreTranslateMessage(pMsg);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::UpdateFont()
{
	chatout.SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);

	TCITEM		tcitem;
	CChatItem	*pChatItem;
	int			i = 0;
	
	tcitem.mask = TCIF_PARAM;
	while (GetItem(i++, &tcitem))
	{
		pChatItem = (CChatItem*)tcitem.lParam;
		pChatItem->m_pLog->SetFont(&g_eMuleApp.m_pdlgEmule->m_fontDefault);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnEnLinkMessageBox(NMHDR *pNMHDR, LRESULT *pResult)
{
	*pResult = 0;

	ENLINK		*pEnLink = reinterpret_cast<ENLINK *>(pNMHDR);

	if (pEnLink && pEnLink->msg == WM_LBUTTONDOWN)
	{
		CString			strUrl;

		chatout.GetTextRange(pEnLink->chrg.cpMin, pEnLink->chrg.cpMax, strUrl);
		CString sLangUrl = strUrl; // GetTextRange returns '?' for non-english chars :(
		sLangUrl.Remove('?'); sLangUrl.Remove(' ');
		if ((strUrl == CLIENT_NAME) || sLangUrl.IsEmpty())
			ShellExecute(NULL, NULL, _T("http://emuleplus.info"), NULL, NULL, SW_SHOWDEFAULT);
		*pResult = 1;
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::OnDestroy()
{
	if (m_Timer)
		KillTimer(m_Timer);
	CClosableTabCtrl::OnDestroy();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CChatSelector::SendAwayMessage(CChatItem* ci)
{
	if (ci == NULL  || ci->m_pClient == NULL || ci->m_pClient->GetChatState() == MS_CONNECTING) //send AwayMessage only if connected
		return false;

#ifdef OLD_SOCKETS_ENABLED
	if (ci->m_pClient->m_pRequestSocket && ci->m_pClient->m_pRequestSocket->IsConnected())
	{
		CString		strAwayMessage = _T("[AUTO REPLY:AWAY] ") + g_eMuleApp.m_pGlobPrefs->GetAwayStateMessage();
		uint16		uMessageLen = _tcslen(static_cast<LPCTSTR>(strAwayMessage));
		Packet		*pPacket = new Packet(OP_MESSAGE, uMessageLen + 2);

		memcpy2(pPacket->m_pcBuffer, &uMessageLen, 2);								// <messagelen 2>
		memcpy2(pPacket->m_pcBuffer + 2, strAwayMessage, uMessageLen);				// (<messagechar 1>)*messagelen
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pPacket->m_dwSize);
		ci->m_pClient->m_pRequestSocket->SendPacket(pPacket, true, true);

		COleDateTime		timelog(COleDateTime::GetCurrentTime());
		CString				strTmp;

		strTmp.Format(_T("%s (%s): "), g_eMuleApp.m_pGlobPrefs->GetUserNick(), timelog.Format());
		ci->m_pLog->AppendText(strTmp, RGB(1, 180, 20));
		ci->m_pLog->AppendText(strAwayMessage + _T('\n'));
	}
#endif //OLD_SOCKETS_ENABLED
	return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CChatSelector::SendMessagePacket(CChatItem &ci, const CString &strMessage)
{
	uint16		uMessageLen = strMessage.GetLength();
	Packet		*pPacket = new Packet(OP_MESSAGE, uMessageLen + 2);

	memcpy2(pPacket->m_pcBuffer, &uMessageLen, 2);							//<messagelen 2>
	memcpy2(pPacket->m_pcBuffer + 2, strMessage.GetString(), uMessageLen);	//(<messagechar 1>)*messagelen
	g_eMuleApp.m_pUploadQueue->AddUpDataOverheadOther(pPacket->m_dwSize);
#ifdef OLD_SOCKETS_ENABLED
	ci.m_pClient->m_pRequestSocket->SendPacket(pPacket, true, true);
#endif //OLD_SOCKETS_ENABLED
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
