//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#ifndef NEW_SOCKETS_ENGINE
#include "stdafx.h"
#endif //NEW_SOCKETS_ENGINE
#include "KnownFile.h"
#include "types.h"
#include "opcodes.h"
#ifndef NEW_SOCKETS_ENGINE
#include "BarShader.h"
#endif //NEW_SOCKETS_ENGINE
#include "SourceSaver.h"
#include "PastComment.h"
#include "Category.h"

//	TEMP - MOREVIT: I don't think the length of the download queue accurately gives the
//		number of transferring sources.
#define USE_TRANSFERRINGSRCCOUNT	1

enum _EnumPartFileStatuses
{
	PS_READY			= 0,
	PS_EMPTY			= 1,
	PS_WAITINGFORHASH	= 2,
	PS_HASHING			= 3,
	PS_ERROR			= 4,
	PS_UNKNOWN			= 6,
	PS_PAUSED			= 7,
	PS_COMPLETING		= 8,
	PS_COMPLETE			= 9,
	PS_STOPPED			= 10,	//eklmn: added stop status
	PS_DOWNLOADING		= 20,	//MOREVIT - Dummy status
	PS_WAITINGFORSOURCE	= 21,	//MOREVIT - Dummy status
	PS_STALLED			= 22,	//MOREVIT - Dummy status
	PS_ACTIVE			= 23	//MOREVIT - Dummy status = Downloading || on queue
};
typedef EnumDomain<_EnumPartFileStatuses>	EnumPartFileStatuses;

//	The number of days without a transfer before a download is considered stalled
#define WARN_PERIOD_OF_NO_PROGRESS 7

#define BUFFER_TIME_LIMIT	180000   // Max milliseconds before forcing a flush // netwolf: increased to 3 minutes (be nice to your HDD :))

// Defines for status of filled parts
#define PART_CORRUPTED				0x01000000
#define PART_VERIFIED				0x02000000
#define NOT_AVAILABLE				0x00FFFFFF

#define PARTIALLY_BLOCKED			0x01
#define FULLY_BLOCKED				0x02
#define FREE_TO_DL					0x00

class CSearchFile;
class CUpDownClient;
class CClientSource;

struct PartFileBufferedData
{
	BYTE				*m_pbyteBuffer;		// This is the data to be written
	uint32				m_dwStartOffset;	// This is the start offset of the data
	uint32				m_dwEndOffset;		// This is the end offset of the data
};

struct PAThreadParam
{
	uint32		iPosition;
	CPartFile*	cPartFile;
};

class CPartFile : public CKnownFile
{
public:
	CPartFile();
	CPartFile(CSearchFile* searchresult, EnumCategories eCatID);
	CPartFile(CString edonkeylink, EnumCategories eCatID);
	CPartFile(class CED2KFileLink* fileLink, EnumCategories eCatID);
	void	InitializeFromLink(CED2KFileLink* fileLink, EnumCategories eCatID);
	virtual	~CPartFile();

//	file status functions (get status)
	EnumPartFileStatuses	GetStatus(bool ignorepause = false);
	void	SetStatus(EnumPartFileStatuses eNewStatus);
	int		GetPartFileStatusID();
	int		GetPartFileStatusRang();
	bool	IsPartFile()				{return !(m_eStatus == PS_COMPLETE);}
	bool	IsPaused() const			{return m_bPaused;}
	bool	IsStopped() const			{return m_bStopped;}
	bool	IsStalled() const;
	bool	IsCompleting() const		{return (m_eStatus == PS_COMPLETING);}
	bool	IsPreallocated() const		{ return m_bIsPreallocated; }
	bool	PreviewAvailable();
	bool	AllowGet1stLast();
	bool	IsArchive(bool bOnlyPreviewable = false);
	bool	IsMovie() const				{ return m_bIsMovie; }
	bool	IsAviMovie() const			{ return m_bIsAviMovie; }
	bool	IsMpgMovie() const			{ return m_bIsMpgMovie; }
	bool	IsMpgAudio() const			{ return m_bIsMpgAudio; }

//	file status functions (set & change status)
	void	DeleteFile();
	void	StopFile(bool bUpdateDisplay = true);
	void	PauseFile();
	void	ResumeFile();
	void	PreviewFile();
	void	AllocateNeededSpace();

//	file property & statistic functions
	uint64	GetRealFileSize();
	uint32	GetSizeToTransfer() const				{ return m_dwGapsSum; }

//	file name & dirs related
	void	SetFileName(const CString& NewName, bool bClearName = true);
	CString	GetFullName() const						{return m_strFullName;}
	CString	GetPartMetFileName() const				{return m_strPartMetFileName;}
	CString	GetTempDir() const						{return m_strTempDir;}
	CString	GetOutputDir();
	void	SetAlternativeOutputDir(CString* path);
	bool	IsAlternativeOutputDir() const			{return !m_strAlternativePath.IsEmpty();}

//	file priority
	bool	IsAutoPrioritized() 					{ return m_bAutoPriority ; }
	void	SetAutoPriority(bool newAutoPriority) 	{ m_bAutoPriority = newAutoPriority ; }
	void	SetPriority(byte byteNewPriority, bool bSaveSettings = true);
	byte	GetPriority()							{return priority;}
	void	UpdateDownloadAutoPriority(void);

//	file comments
	bool	HasComment() const		{return hasComment;}
	void	SetHasComment(bool in)			{hasComment=in;}
	void	AddPastComment(CUpDownClient *pClient);
	void	RemovePastComment(CUpDownClient *pClient, bool bRestore= true);
	CPastCommentList &GetPastCommentList()			{ return m_pastCommentList; }

//	file category
	EnumCategories	GetCatID() const;
	void			SetCatID(EnumCategories eCatID);

//	file rating
	EnumPartFileRating		GetRating();
	bool	HasRating() const			{return hasRating;}
	void	SetHasRating(bool in)			{hasRating=in;}
	void	UpdateFileRatingCommentAvail();


//	part status function (get functions)
	bool	IsComplete(uint32 start, uint32 end);
	bool	IsPartComplete(uint32 dwPart) const;
	bool	IsPartComplete(uint32 dwPart, uint32 *pdwStart, uint32 *pdwEnd) const;
	bool	IsPartFull(uint32 dwPart) const;
	bool	IsPartDownloading(uint32 dwPart) const;
	bool	IsCorruptedPart(uint32 dwPartNum) const {return ((PART_CORRUPTED & m_PartsStatusVector[dwPartNum]) != 0);}

	uint32	GetPartFilledSize(uint32 dwPartNum) const;
	uint32	GetPartLeftToDLSize(uint32 dwPartNum) const {return (0x00FFFFFF & m_PartsStatusVector[dwPartNum]);}

	uint16	GetNextRequiredPart(CUpDownClient* sender);
	uint16	GetSrcPartFrequency(uint16 part) { return m_srcPartFrequencies[part]; }
	uint16	GetRemainingBlocksInPart(uint16 partnumber);
	uint32	GetAvailablePartCount() const	{return m_dwAvailablePartsCount;}
	void	GetFirstLastChunk4Preview();

//	part status functions (set & change status)
	void	PartFileHashFinished(CKnownFile* result);
	bool	HashSinglePart(uint32 dwPartNum); // true = ok , false = corrupted
	void	NewSrcPartsInfo();

//	source functions
	void	AddClientSource(CClientSource* pSource, int iSource=0, bool bExchanged=false, byte byteSourceExchangeVersion = 0);
	void	AddClientSources(CTypedPtrList<CPtrList, CClientSource*>*);
	void	AddClientSources(CMemFile* sources, byte sourceexchangeversion = 1);
	void	AddSources(CMemFile& pSources, uint32 iServerIP, uint16 iServerPort);
	virtual	Packet* CreateSrcInfoPacket(CUpDownClient* ForClient);
	void	GetSourcesAfterServerConnect();
	void	RemoveNoNeededSources(uint32 number_sources_to_remove = 0xffffffff);
	void	RemoveAllSources(bool bTryToSwap);
	void	DownloadAllA4AF(bool bSameCat = false);
	void	AddDownloadingSource(CUpDownClient* client);
	void	RemoveDownloadingSource(CUpDownClient* client);
	void	GetCompleteSourcesRange(uint16 *lo, uint16 *hi)	{*lo= *hi= GetCompleteSourcesCount();}

//	source statistic functions
	uint16	GetSourceCount();
#if USE_TRANSFERRINGSRCCOUNT
	uint16	GetTransferringSrcCount()				{ return m_iNumTransferringSrcs; }
#else
	uint16	GetTransferringSrcCount()				{return m_downloadingSourceList.GetCount();}
#endif
	uint16	GetNoNeededPartsSrcCount()				{return m_iSrcNNP;}
	uint16	GetOnQueueSrcCount()					{return m_iSrcOnQueue;}
	uint16	GetHighQRSrcCount()						{return m_iSrcHighQR;}
	uint16	GetConnectingSrcCount()					{return m_iSrcConnecting;}
	uint16	GetTooManyConnsSrcCount()				{return m_iSrcTooManyConns;}
	uint16	GetConnectedSrcCount()					{return m_iSrcConnected;}
	uint16	GetTimeoutedSrcCount()					{return m_iSrcTimeout ;}
	uint16	GetConnectingViaServerSrcCount()		{return m_iSrcConnViaServer;}
	uint16	GetUnknownSrcCount()					{return m_iSrcUnknown;}
	uint16	GetRecHashsetSrcCount()					{return m_iSrcRecHash;}
	uint16	GetLow2LowSrcCount()					{return m_iSrcLowToLow;}
	uint16	GetQueueFullSrcCount()					{return m_iSrcQueueFull;}
	uint16	GetNoUploadSrcCount()					{return m_iSrcNoUpload;}
	uint16	GetBannedSrcCount()						{return m_iSrcBanned;}
	uint16	GetErrorSrcCount()						{return m_iSrcError;}
	uint16	GetSrcA4AFCount()						{return m_iSrcA4AF;}
	uint16	GetNotCurrentSourcesCount();
	int		GetValidSourcesCount();
	uint16	GetCompleteSourcesCount()				{return m_uLastCompleteSrcCount;}

//	gap functions
	void	AddGap(uint32 start, uint32 end);
	void	FillGap(uint32 start, uint32 end);
	uint32	GetGapsInPart(uint16 uPartNum, Requested_Block_Struct** ppNewBlocks, uint32 dwCount);
	// Barry - This will invert the gap list, up to caller to delete gaps when done
	// 'Gaps' returned are really the filled areas, and guaranteed to be in order
	void	GetFilledList(CTypedPtrList<CPtrList, Gap_Struct*> *filled);

//	block functions
	bool	IsLastBlockComplete();
	void	RemoveBlockFromList(uint32 start,uint32 end);
	// Barry - Added to prevent list containing deleted blocks on shutdown
	void	RemoveAllRequestedBlocks(void);

#define PF_WR2BUF_FL_FREEBUFFER		0x01
#define PF_WR2BUF_FL_ENDOFBLOCK		0x02
	uint32	WriteToBuffer(sint32 iCompressionGain, BYTE *data, uint32 start, uint32 end, int iFlags);

//	time function
	void	SetStartTimeReset(bool bEnable)		{m_StartTimeReset = bEnable;}
	bool	GetStartTimeReset()					{return m_StartTimeReset;}
	CTimeSpan GetFlushTimeSpan()			{return (m_timeLastDownTransfer - m_SessionStartTime);}
	CTimeSpan GetSessionTimeSpan()			{return (CTime::GetCurrentTime() - m_SessionStartTime);}
	uint32	GetLastPurgeTime()					{ return m_dwLastPurgeTime; }
	sint32	GetTimeRemaining(bool bAvgerage = false);
	uint32	GetLastAnsweredTime()			{ return m_ClientSrcAnswered; }
	CTime	GetLastDownTransfer() const			{ return m_timeLastDownTransfer; }
	void	SetLastAnsweredTime()			{ m_ClientSrcAnswered = ::GetTickCount(); }
	void	SetLastAnsweredTimeTimeout()	{ m_ClientSrcAnswered = 2 * CONNECTION_LATENCY + ::GetTickCount() - SOURCECLIENTREASK; }


//	File statistic (session & cumulative)
	uint32	GetTransferred() const					{ return m_dwBytesTransferred; }
	uint32	GetCompletedSize() const				{ return m_dwCompletedSize; }
	uint32	GetDataRate()							{return m_dwDataRate;}
	void	AddRxAmount(uint32 dwRxBytes)			{ m_dwBytesTransferred += dwRxBytes; }
	void	AddRxCorruptedAmount(uint32 dwBadBytes);
	uint32	GetAvgDataRate(bool bUpdated = false);
	uint32	GetSessionTransferred()					{ return ((m_dwCompletedSize > m_SessionStartSize) ? (m_dwCompletedSize - m_SessionStartSize) : 0); }
	uint64	GetLostDueToCorruption() const			{return m_qwLostDueToCorruption;}
	uint64	GetGainDueToCompression() const			{return m_qwGainDueToCompression;}
	uint64	GetSessionGainDueToCompression() const	{return m_qwSessionGainDueToCompression;}
	uint32	TotalPacketsSavedDueToICH() const		{return m_iTotalPacketsSavedDueToICH;}
	void	UpdateCompletedInfos();
	double	GetPercentCompleted() const				{return m_dblPercentCompleted;}
//	Don't return more than 99.99 for 2 digits display as it will be rounded to 100.00 during number to string conversion
	double	GetPercentCompleted2() const			{return ((m_dblPercentCompleted > 99.99) && (m_dblPercentCompleted < 100.0)) ? 99.99 : m_dblPercentCompleted;}

//	I/O functions
	void	FlushBuffer(void);
	CFile	&GetPartFileHandle()				{return m_hPartFileRead;}
	bool	CreateFromFile(char* directory,char* filename)	{return false;}
	bool	LoadFromFile(FILE* file)						{return false;}
	bool	WriteToFile(FILE* file)							{return false;}
	bool	LoadPartFile(LPCTSTR in_directory, LPCTSTR filename);
	bool	TryToRecoverPartFile(LPCTSTR in_directory, LPCTSTR in_filename);
	bool	SavePartFile();
	void	WritePartStatus(CFile* file);
	void	WriteCompleteSourcesCount(CFile* file);
	void	LoadSettingsFile();
	void	SaveSettingsFile();
	bool	SavePartFileStats();
	int	ReadFileForUpload(uint32 dwOffset, uint32 dwBytesToRead, byte *pbyteBuffer);


//	GUI related functions
	void	DrawStatusBar(CDC* dc, RECT* rect, bool bFlat);
	void	UpdateDisplayedInfo();
	CString	GetPartFileStatus();
	CString	GetProgressString(uint16 size);
	void	CharFillRange(CString* buffer,double start, double end, char color);
	CString	GetDownloadFileInfo();
	CString	GetDownloadFileInfo4Tooltips();
	CString	LocalizeLastSeenComplete()	{return DtLocale(lastseencomplete);}
	CString	LocalizeLastDownTransfer()	{return DtLocale(m_timeLastDownTransfer);}

//	unsorted
	uint32	Process(uint32 dwReduceDownload, uint32 dwIteration);
	uint32	GetCommonFilePenalty();
	byte*	MMCreatePartStatus();
	static DWORD WINAPI AllocateNeededSpaceProc(LPVOID lpParameter);
	CString	CreateED2KSourceLink(uint32 expirein, int sourcenum, bool bIsSourceSaver = false);

	bool	IsFakesDotRar() const						{ return m_bIsFakesDotRar; }
	void	SetFakesDotRar(bool bIsFakesDotRar = true)	{ m_bIsFakesDotRar = bIsFakesDotRar; }

protected:
	bool	IsAlreadyRequested(uint32 start, uint32 end);
	void	CompleteFile(bool hashingdone);
	void	CreatePartFile();
	void	Init();

private:
	CString DtLocale(CTime dt);
	BOOL	PerformFileComplete();
	static UINT CompleteThreadProc(CPartFile* pFile);
	CString GetStatsFullPath() const;
	bool	LoadPartFileStats();
	void	RemovePartFileStats();
	void	ClosePartFile();
	void	UpdateGUIAfterStateChange();
	void	GetGapListCopy(CArray<Gap_Struct, Gap_Struct> *pGapArray);
	void FillGapInParts(uint32 dwGapStart, uint32 dwGapEnd);
	void AddGapToParts(uint32 dwGapStart, uint32 dwGapEnd);
	void	InvalidateRemoteQueueRankingStats()	{ m_uHighestRemoteQueueRank = 0; m_uLowestRemoteQueueRank = 0xFFFF; }

// --- variable sections ---

public:
	CTypedPtrList<CPtrList, CUpDownClient*> m_srcLists[SOURCESSLOTS];
	CTypedPtrList<CPtrList, CUpDownClient*> m_A4AFsrcList;

	// Barry - Is archive recovery in progress
	volatile bool	m_bRecoveringArchive;
	volatile bool	m_bPreviewing;

	bool		m_bHashSetNeeded;
	CTime		lastseencomplete;

protected:
	CSourceSaver		m_sourcesaver;

private:
//	Separate read and write handles to eliminate I/O collisions
	CFile	m_hPartFileWrite;
	CFile	m_hPartFileRead;

	EnumCategories	m_eCategoryID;
	bool			m_bDataFlushReq;
	uint16			m_uProcessCounter;
//	Statistics
//	We don't need to count transferring sources, because we can get them over m_downloadingSourceList.GetCount()
	uint16	m_iNumTransferringSrcs;
	uint16	m_iSrcNNP;
	uint16	m_iSrcOnQueue;
	uint16	m_iSrcHighQR;
	uint16	m_iSrcConnecting;
	uint16	m_iSrcTooManyConns;
	uint16	m_iSrcConnected;
	uint16	m_iSrcTimeout;
	uint16	m_iSrcConnViaServer;
	uint16	m_iSrcUnknown;
	uint16	m_iSrcRecHash;
	uint16	m_iSrcLowToLow;
	uint16	m_iSrcQueueFull;
	uint16	m_iSrcBanned;
	uint16	m_iSrcNoUpload;
	uint16	m_iSrcError;
	uint16	m_iSrcA4AF;
	uint32	m_dwCompletedSize;
	uint64	m_qwLostDueToCorruption;
	uint64	m_qwGainDueToCompression;
	uint64	m_qwSessionGainDueToCompression;
	uint32	m_iTotalPacketsSavedDueToICH;
	uint32	m_dwDataRate;
	CString	m_strFullName;
	CString	m_strTempDir;
	CString	m_strPartMetFileName;
	uint32	m_dwBytesTransferred;
	bool	m_bPaused;
	bool	m_bStopped;
	byte	priority;
	EnumPartFileStatuses	m_eStatus;
	uint32	m_dwLastFileSourcesRequestTime;
	uint32	m_dwLastPurgeTime;
	uint32	m_LastNoNeededCheck;

	CTypedPtrList<CPtrList, Gap_Struct*> gaplist;
	CTypedPtrList<CPtrList, Requested_Block_Struct*> requestedblocks_list;
	CArray<uint16,uint16> m_srcPartFrequencies;
	uint16	m_uMaxSrcPartFrequency;
	std::vector<uint32> m_PartsStatusVector;
	CRITICAL_SECTION					m_csDownloadingSourceList;
	CTypedPtrList<CPtrList, CUpDownClient*> m_downloadingSourceList;
	double	m_dblPercentCompleted;
	uint32	m_dwAvailablePartsCount;
	uint32	m_ClientSrcAnswered;
	bool	m_bAutoPriority;
	bool	m_bIsBeingDeleted;
	bool	m_bIsPreallocated;

	CTime	m_SessionStartTime;
	uint32	m_SessionStartSize;
	uint32	m_AvgDataRate;
	uint32	m_dwCompletedPartsSize;
	bool	m_StartTimeReset;

	bool	m_bIsFakesDotRar;
	bool	hasRating;
	bool	hasComment;

	CTypedPtrList<CPtrList, PartFileBufferedData*> m_BufferedData_list;
	uint32	m_nTotalBufferData;
	uint32	m_nLastBufferFlushTime;
	uint16	m_uLastCompleteSrcCount;

	CCriticalSection	m_csSavePartFile;
	CCriticalSection	m_csGapListAndPartStatus;
	CCriticalSection	m_csFileCompletion;

	CTime	m_timeLastDownTransfer;
	CString	m_strAlternativePath;

	CPastCommentList m_pastCommentList;

	uint32	m_dwGapsSum;

//	Highest QR (0=invalid) is used for precise source dropping
	uint16	m_uHighestRemoteQueueRank;
//	Lowest QR (0xFFFF=invalid) is currently only calculated but maybe used for the feature request #291 later
	uint16	m_uLowestRemoteQueueRank;

	bool	m_bIsPreviewableArchive;
	bool	m_bIsMovie;
	bool	m_bIsAviMovie;
	bool	m_bIsMpgMovie;
	bool	m_bIsMpgAudio;
};
