//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "UDPSocket.h"
#include "otherfunctions.h"
#include "otherstructs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

#define	ERR_UDP_MISCONFIGURED_SERVER _T("Error while processing incoming UDP Packet (most likely a misconfigured server)")

CUDPSocketWnd::CUDPSocketWnd(){
}

BEGIN_MESSAGE_MAP(CUDPSocketWnd, CWnd)
	ON_MESSAGE(WM_DNSLOOKUPDONE, OnDNSLookupDone)
END_MESSAGE_MAP()

LRESULT CUDPSocketWnd::OnDNSLookupDone(WPARAM wParam,LPARAM lParam)
{
	EMULE_TRY

	if(m_pOwner)
		m_pOwner->DnsLookupDone(wParam,lParam);
	return true;

	EMULE_CATCH

	return false;
};
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUDPSocket::CUDPSocket(CServerConnect* in_serverconnect)
{
	m_hWndResolveMessage = NULL;
	m_pcSendBuffer = 0;
	m_pServer = 0;
	m_pServerConnect = in_serverconnect;
	DnsTaskHandle = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUDPSocket::~CUDPSocket()
{
	EMULE_TRY

	if (m_pServer)
		delete m_pServer;
	if (m_pcSendBuffer)
		delete[] m_pcSendBuffer;

	m_udpwnd.DestroyWindow();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool  CUDPSocket::Create()
{
	EMULE_TRY

	VERIFY( m_udpwnd.CreateEx(0, AfxRegisterWndClass(0),_T("Emule Socket Wnd"),WS_OVERLAPPED, 0, 0, 0, 0, NULL, NULL));
	m_hWndResolveMessage = m_udpwnd.m_hWnd;
	m_udpwnd.m_pOwner = this;

	return CAsyncSocket::Create(0,SOCK_DGRAM,FD_READ);

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::OnReceive(int nErrorCode)
{
	EMULE_TRY

	byte		abyteBuffer[5000];
	CString		serverbuffer;
	UINT		uiUDPPort;
	int			iLength = ReceiveFrom(abyteBuffer, sizeof(abyteBuffer), serverbuffer, uiUDPPort);

	if ((iLength >= 2) && (abyteBuffer[0] == OP_EDONKEYPROT))	//SOCKET_ERROR = -1
		ProcessPacket(abyteBuffer + 2, iLength - 2, abyteBuffer[1], serverbuffer, uiUDPPort);
	
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUDPSocket::ProcessPacket(byte *pbytePacket, uint32 dwSize, byte uOpcode, LPCTSTR host, uint16 uUDPPort)
{
	CServer	*pServer = NULL;

	try
	{
		g_eMuleApp.m_pDownloadQueue->AddDownDataOverheadServer(dwSize);
		if ((pServer = g_eMuleApp.m_pServerList->GetServerByAddress(host, uUDPPort - 4)) == NULL)
		{
		//	Don't process packet from unknown servers
			return false;
		}
		pServer->ResetFailedCount();
		g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);

		switch (uOpcode)
		{
			case OP_GLOBSEARCHRES:
			{
				CSafeMemFile	pckStream(pbytePacket, dwSize);
				uint32			dwLeft;
				byte			byteTmp;

				do
				{
					g_eMuleApp.m_pdlgEmule->m_dlgSearch.AddUDPResult(g_eMuleApp.m_pSearchList->ProcessUDPSearchAnswer(pckStream));

				//	Check if there is another search results packet
					dwLeft = pckStream.GetLength() - pckStream.GetPosition();
					if (dwLeft >= 2)
					{
						pckStream.Read(&byteTmp, 1);
						dwLeft--;
						if (byteTmp != OP_EDONKEYPROT)
							break;

						pckStream.Read(&byteTmp, 1);
						dwLeft--;
						if (byteTmp != OP_GLOBSEARCHRES)
							break;
					}
				} while(dwLeft != 0);
				break;
			}
			case OP_GLOBFOUNDSOURCES:
			{
				CSafeMemFile	sources((BYTE*)pbytePacket, dwSize);
				uchar			abyteFileId[16];
				uint32			dwLeft;
				byte			byteTmp;

				do
				{
					sources.Read(abyteFileId, 16);

					CPartFile* 		pPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(abyteFileId);

					if (pPartFile)
						pPartFile->AddSources(sources, inet_addr(host), uUDPPort - 4);
					else
					{
					//	Skip sources for that file
						sources.Read(&byteTmp, 1);
						sources.Seek(byteTmp * (4 + 2), SEEK_CUR);
					}

				//	Check if there is another source packet
					dwLeft = sources.GetLength() - sources.GetPosition();
					if (dwLeft >= 2)
					{
						sources.Read(&byteTmp, 1);
						dwLeft--;
						if (byteTmp != OP_EDONKEYPROT)
							break;

						sources.Read(&byteTmp, 1);
						dwLeft--;
						if (byteTmp != OP_GLOBFOUNDSOURCES)
							break;
					}
				} while(dwLeft != 0);
				break;
			}
 			case OP_GLOBSERVSTATRES:
			{
			//	Check minimal allowed size (contain basic information)
				if (dwSize < 12)
					return false;

				if (PEEK_DWORD(pbytePacket) != pServer->GetChallenge())
					return true; 
				pServer->SetChallenge(0);

				pServer->SetUserCount(PEEK_DWORD(pbytePacket + 4));
				pServer->SetFileCount(PEEK_DWORD(pbytePacket + 8));

				pServer->SetMaxUsers((dwSize >= 16) ? PEEK_DWORD(pbytePacket + 12) : 0);
				pServer->SetSoftMaxFiles((dwSize >= 24) ? PEEK_DWORD(pbytePacket + 16) : 0);
				pServer->SetHardMaxFiles((dwSize >= 24) ? PEEK_DWORD(pbytePacket + 20) : 0);

				pServer->SetUDPFlags((dwSize >= 28) ? PEEK_DWORD(pbytePacket + 24) : 0);
				pServer->SetLowIDUsers((dwSize >= 32) ? PEEK_DWORD(pbytePacket + 28) : 0);

				pServer->SetPing(::GetTickCount() - pServer->GetLastPinged());
				g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
				break;
			}

 			case OP_SERVER_DESC_RES:
			{
				// old packet: <name_len 2><name name_len><desc_len 2><desc desc_len>
				// new packet: <challenge 4><taglist>
				//
				// NOTE: To properly distinguish between the two packets which are both using the same opcode...
				// the first two bytes of <challenge> (in network byte order) have to be an invalid <name_len> at least.

				CSafeMemFile	srvinfo((BYTE*)pbytePacket, dwSize);
				uint32			ui, dwTmp;

				if ((dwSize >= 8) && (PEEK_WORD(pbytePacket) == INV_SERV_DESC_LEN))
				{
					srvinfo.Read(&dwTmp, 4);	 // read challenge
					if ((pServer->GetDescReqChallenge() != 0) && (dwTmp == pServer->GetDescReqChallenge()))
					{
						pServer->SetDescReqChallenge(0);
						srvinfo.Read(&dwTmp, 4);	 // read tag count
						for (ui = 0; ui < dwTmp; ui++)
						{
							CTag	TempTag;

							TempTag.FillFromStream(srvinfo);
							if (TempTag.GetTagID() == ST_SERVERNAME)
							{
								if (TempTag.IsStr() && !pServer->IsStaticMember())
									pServer->SetListName(TempTag.GetStringValue());
							}
							else if (TempTag.GetTagID() == ST_DESCRIPTION)
							{
								if (TempTag.IsStr())
									pServer->SetDescription(TempTag.GetStringValue());
							}
							else if (TempTag.GetTagID() == ST_DYNIP)
							{
								if (TempTag.IsStr())
									pServer->SetDynIP(TempTag.GetStringValue());
							}
							else if (TempTag.GetTagID() == ST_VERSION)
							{
								if (TempTag.IsStr())
									pServer->SetVersion(TempTag.GetStringValue());
								else if (TempTag.IsInt())
								{
									CString strVersion;

									strVersion.Format(_T("%u.%u"), TempTag.GetIntValue() >> 16, TempTag.GetIntValue() & 0xFFFF);
									pServer->SetVersion(strVersion);
								}
							}
							else if (TempTag.GetTagID() == ST_AUXPORTSLIST)
							{
								if (TempTag.IsStr())	// <string> = <port> [, <port>...]
								{
								}
							}
						}
					}
				}
				else
				{
					uint16	uStringLen;
					CString	strBuf;

					srvinfo.Read(&uStringLen, 2);
					srvinfo.Read(strBuf.GetBuffer(uStringLen), uStringLen);
					strBuf.ReleaseBuffer(uStringLen);

					if(!pServer->IsStaticMember())
						pServer->SetListName(strBuf);

					srvinfo.Read(&uStringLen, 2);
					srvinfo.Read(strBuf.GetBuffer(uStringLen), uStringLen);
					strBuf.ReleaseBuffer(uStringLen);

					pServer->SetDescription(strBuf);
				}
				g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
				break;
			}
			default:
				return false;
		}
		return true;
	}
	catch(CFileException* error)
	{
		CString	strBuff(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED_SERVER);

		if (pServer != NULL)
			strBuff.AppendFormat(_T(" %s:%u"), pServer->GetAddress(), pServer->GetPort());

		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(strBuff);
		error->Delete();
		if (uOpcode == OP_GLOBSEARCHRES || uOpcode == OP_GLOBFOUNDSOURCES)
			return true;
	}
	catch(CMemoryException* error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED_SERVER _T(" - CMemoryException"));
		error->Delete();
		if (uOpcode == OP_GLOBSEARCHRES || uOpcode == OP_GLOBFOUNDSOURCES)
			return true;
	}
	catch(CString error)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED_SERVER _T(" - ") + error);
	}
	catch(...)
	{
		OUTPUT_DEBUG_TRACE();
		AddDebugLogLine(RGB_LOG_ERROR_TXT ERR_UDP_MISCONFIGURED_SERVER);
	}
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::AsyncResolveDNS(LPCTSTR lpszHostAddress, UINT nHostPort)
{
	EMULE_TRY

	m_lpszHostAddress = lpszHostAddress;
	m_nHostPort = nHostPort;
	if (DnsTaskHandle)
		WSACancelAsyncRequest(DnsTaskHandle);
	DnsTaskHandle = NULL;
// 	See if we have an IP already
	USES_CONVERSION;
	SOCKADDR_IN sockAddr;
	memzero(&sockAddr, sizeof(sockAddr));
	LPSTR lpszAscii = T2A((LPTSTR)m_lpszHostAddress);
	sockAddr.sin_family = AF_INET;
	sockAddr.sin_addr.s_addr = inet_addr(lpszAscii);
	sockAddr.sin_port = htons((u_short)m_nHostPort);

//	Backup for send socket
	m_SaveAddr = sockAddr;

	if (sockAddr.sin_addr.s_addr == INADDR_NONE)
	{
	//	Resolve hostname "hostname" asynchronously
		memzero(DnsHostBuffer, sizeof(DnsHostBuffer));

		USES_CONVERSION;
		DnsTaskHandle = WSAAsyncGetHostByName( m_hWndResolveMessage,
											   WM_DNSLOOKUPDONE,
											   CT2A(lpszHostAddress),
											   DnsHostBuffer,
											   MAXGETHOSTSTRUCT );

		if (DnsTaskHandle == 0)
		{
			delete[] m_pcSendBuffer;
			m_pcSendBuffer = 0;
			delete m_pServer;
			m_pServer = 0;
#ifdef _DEBUG
			AfxMessageBox(_T("LOOKUPERROR DNSTASKHANDLE = 0"));
#endif
		}
	}
	else
	{
		SendBuffer();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::DnsLookupDone(WPARAM wp, LPARAM lp)
{
	EMULE_TRY

	DnsTaskHandle = NULL;

//	An asynchronous database routine completed
	if (WSAGETASYNCERROR(lp) != 0)
	{
		if (m_pcSendBuffer)
			delete[] m_pcSendBuffer;
		m_pcSendBuffer = 0;
		if (m_pServer)
			delete m_pServer;
		m_pServer = 0;
		return;
	}
	if (m_SaveAddr.sin_addr.s_addr == INADDR_NONE)
	{
	//	Get the structure length
		int iBufLen = WSAGETASYNCBUFLEN(lp);
		LPHOSTENT lphost = (LPHOSTENT)malloc(iBufLen);
		memcpy2(lphost, DnsHostBuffer, iBufLen);
		m_SaveAddr.sin_addr.s_addr = ((LPIN_ADDR)lphost->h_addr)->s_addr;
		free(lphost);
	//	Also reset the receive buffer
		memzero(DnsHostBuffer, sizeof(DnsHostBuffer));
	}
	if (m_pServer)
	{
		CServer* update = g_eMuleApp.m_pServerList->GetServerByAddress(m_pServer->GetAddress(),m_pServer->GetPort());
		if (update)
			update->SetID(m_SaveAddr.sin_addr.S_un.S_addr);
		SendBuffer();
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::SendBuffer()
{
	EMULE_TRY

	if(m_pServer && m_pcSendBuffer)
	{
		SendTo(m_pcSendBuffer,m_dwSendBufLength,(SOCKADDR*)&m_SaveAddr, sizeof(m_SaveAddr));
	}
	safe_delete(m_pServer);
	safe_delete(m_pcSendBuffer);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUDPSocket::SendPacket(Packet *pPacket,CServer *pHostServer)
{
//	At this point m_pServer & m_pcSendBuffer must be already NULL but it seems that sometime we are not able
//	to get WM_DNSLOOKUPDONE before we send a new packet therefore the existency of the objects will be checked
	safe_delete(m_pServer);
	safe_delete(m_pcSendBuffer);

//	Now we are trying to create a new objects
	try
	{
		m_pServer = new CServer(*pHostServer);
		m_pcSendBuffer = new char[pPacket->m_dwSize + sizeof(UDP_Header_Struct)];
		memcpy(m_pcSendBuffer, pPacket->GetUDPHeader(), sizeof(UDP_Header_Struct));
		memcpy2(m_pcSendBuffer + sizeof(UDP_Header_Struct), pPacket->m_pcBuffer, pPacket->m_dwSize);
		m_dwSendBufLength = pPacket->m_dwSize + sizeof(UDP_Header_Struct);
		AsyncResolveDNS(m_pServer->GetAddress(), m_pServer->GetPort() + 4);
	}
	catch (CException * error)
	{
		OUTPUT_DEBUG_TRACE();
		error->Delete();
		safe_delete(m_pServer);
		safe_delete(m_pcSendBuffer);
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif //OLD_SOCKETS_ENABLED
